/*------------------------------------------------------------------------------
 *                          Silicon Laboratories, Inc.
 *                           http://www.silabs.com
 *                               Copyright 2010
 *------------------------------------------------------------------------------
 *
 *    FILE:       aes_demo_main.c
 *    TARGET:     Si4010
 *    TOOLCHAIN:  Generic
 *    DATE:       May 05, 2010, Wednesday
 *    RELEASE:    2.1 (MPech), ROM revision 02.00, trim 3
 *
 *------------------------------------------------------------------------------
 *
 *    DESCRIPTION:
 *      This file contains the main function for the Keil toolchain
 *      sample project.
 *
 *      BL51 linker directives for building this application:
 *
 *      BL51: PL(68) PW(78) IXREF RS(256)
 *            CODE (0x0-0x0DFF)
 *            DATA (0x40)
 *            IDATA (0x80)
 *            XDATA (0x0E00-0x107F)
 *            STACK (?STACK(0x90))
 *
 *------------------------------------------------------------------------------
 *
 *    INCLUDES:
 */
#include "si4010.h"
#include "si4010_api_rom.h"

#include "si4010_types.h"
#include "aes_demo.h"
/*------------------------------------------------------------------------------
 *
 *    FUNCTION DESCRIPTION:
 *      Main function for the example.
 *
 *------------------------------------------------------------------------------
 */

void    main(void)
{
//Disable the Matrix and Roff modes 
  PORT_CTRL &= ~(M_PORT_MATRIX | M_PORT_ROFF | M_PORT_STROBE);
  PORT_CTRL |=  M_PORT_STROBE;
  PORT_CTRL &= (~M_PORT_STROBE);

//Turn LED control off 
  GPIO_LED = 0;

//Call to API. Expected result format is BCD:
//   wRomId  .. 0x0200 == 02.00
 
  wRomId = wSys_GetRomId();

//-- Test the AES 

//Initialize the Timer 2 .. TMR2 
  TMR2CTRL = 0x00;    //16 bit wide timer 
  TMR_CLKSEL = 0x00;  //Default value, clk_sys as a source frequency 

//Clear Timer 2 
  wEncTimer = 0;
  wDecTimer = 0;
  wKeyTimer = 0;
  wKeyDecTotal = 0;

  TMR2L_RUN = 0;  //Stop TMR2 
  TMR2L = 0x00;
  TMR2H = 0x00;

  TMR2RL = 0x00;  //Reload values 
  TMR2RH = 0x00;

//Copy input key and data from CODE to IDATA to work on 
  vCopyInputCode( abPlainData, abState );
  vCopyInputCode( abKey, abRoundKey );

//Clear the output .. use the round key as random data to clear
// the output in XDATA. 
  vCopyOutputXdata( abRoundKey, abEncryptedData );

//Encrypt and measure time in system clock cycles 
  TMR2L_RUN = 1;  //Start Timer 2 
  vAes_Cipher( abState, abRoundKey );
  TMR2L_RUN = 0;  //Stop Timer 2 

//Calculate overall number of system clock cycles for encryption 
  wEncTimer = TMR2H * 256 + TMR2L;

//Copy the encrypted data to the output array in XDATA 
  vCopyOutputXdata( abState, abEncryptedData );

//Check data .. if bResult != 0 then error 
  bResult = bCheckData( abEncryptedData, abCipherData );

//Clear TMR2 
  TMR2CTRL &= ~(M_TMR2INTH | M_TMR2INTL);
  TMR2L = 0x00;
  TMR2H = 0x00;

//Clear the output .. use the round key as random data 
  vCopyOutputXdata( abRoundKey, abDecryptedData );

//Decryption .. inverse cipher. Copy the encryption key. 
  vCopyInputCode( abKey, abRoundKey );

//Prepare the decryption key from the encryption key.
// Measure how many system clock cycles it takes. 
  TMR2L_RUN = 1;  //Start Timer 2 
  vAes_InvGenKey( abRoundKey );
  TMR2L_RUN = 0;  //Stop Timer 2 

//Calcluate the ovearall decryption key geneation time 
  wKeyTimer = TMR2H * 256 + TMR2L;

//Clear TMR 2 
  TMR2CTRL &= ~(M_TMR2INTH | M_TMR2INTL);
  TMR2L = 0x00;
  TMR2H = 0x00;

//Decrypt and measure time in system clock cycles 
  TMR2L_RUN = 1;  //Start Timer 2 
  vAes_InvCipher( abState, abRoundKey );
  TMR2L_RUN = 0;  //Stop Timer 2 
  wDecTimer = TMR2H * 256 + TMR2L;

//Total decryption time including the key generaiton 
  wKeyDecTotal = wKeyTimer + wDecTimer;

//Copy output data .. the decrypted data should be the
// same as the input abPlainData[
  vCopyOutputXdata( abState, abDecryptedData );

//Check data .. if bResult != 0 then error 
  bResult = bCheckData( abDecryptedData, abPlainData );

//-- Toggle GPIO[1] 
  GPIO1 = 1;
  GPIO1 = 0;

//Toggle GPIO[2] 
  GPIO2 = 1;
  GPIO2 = 0;

//Inifinite loop 
  while ( 1 ) {};

//This will never happen. 
  return;
}
/*-------------------------------------------------------------
 *
 *    FUNCTION DESCRIPTION:
 *      Copy data or key from Code to State.
 *
 *-------------------------------------------------------------
 */

static void vCopyInputCode(BYTE const code *pbiSource,BYTE idata *pboData)
{
/*
 *-------------------------------------------------------------
 *
 *    VARIABLES:
 */
  BYTE          bInd;           //Loop index 

/*
 *-------------------------------------------------------------
 */

//-- Copy data 
  for ( bInd = 16; bInd > 0; bInd-- )
  {
    *pboData = *pbiSource;
    pbiSource++;
    pboData++;
  }

//-- Return 
  return;
}
/*-------------------------------------------------------------
 *
 *    FUNCTION DESCRIPTION:
 *      Check byte by byte 16 byte data
 *      Return:
 *        0 .. match
 *        1 .. error
 *
 *-------------------------------------------------------------
 */

static BYTE bCheckData(BYTE xdata *pbiOutData,BYTE const code *pbiInData)
{
/*
 *-------------------------------------------------------------
 *
 *    VARIABLES:
 */
  BYTE          bInd;           //Loop index 
  BYTE          bRetVal;

/*
 *-------------------------------------------------------------
 */

//-- Copy data 
  bRetVal = 0;  //Success 

  for ( bInd = 16; bInd > 0; bInd-- )
  {
    if ( *pbiOutData != *pbiInData )
    {
      bRetVal = 1;
      break;
    }
    pbiOutData++;
    pbiInData++;
  }

//-- Return 
  return ( bRetVal );
}
/*-------------------------------------------------------------
 *
 *    FUNCTION DESCRIPTION:
 *      Copy output to XDATA.
 *
 *-------------------------------------------------------------
 */

static void vCopyOutputXdata(BYTE idata *pbiData,BYTE xdata *pboDest)
{
/*
 *-------------------------------------------------------------
 *
 *    VARIABLES:
 */
  BYTE          bInd;           //Loop index 

/*
 *-------------------------------------------------------------
 */

//-- Copy data 
  for ( bInd = 16; 0 != bInd; bInd-- )
  {
    *pboDest = *pbiData;
    pbiData++;
    pboDest++;
  }

//-- Return 
  return;
}

/*
 *------------------------------------------------------------------------------
 */

