/*------------------------------------------------------------------------------
 *                          Silicon Laboratories, Inc.
 *                           http://www.silabs.com
 *                               Copyright 2010
 *------------------------------------------------------------------------------
 *
 *    FILE:       OD_main.c
 *    TARGET:     Si4010
 *    TOOLCHAIN:  Keil
 *    RELEASE:    1.1 ; ROM version 02.00
 *    AUTHOR:     Miklos Lukacs
 *    DATE:       Jun 07, 2010
 *    EDITOR:     Viktor Huszar
 *    DATE:       Aug 12, 2011 (Add: BMSR)
 *    DATE:       Jun 03, 2013 (Add: BMSR test)
 *
 *------------------------------------------------------------------------------
 *
 *    DESCRIPTION:
 *      This file contains the main function for the Keil toolchain
 *      sample keyfob_demo project.
 *
 *      BL51 linker directives for building this application:
 *
 *      BL51: PL(68) PW(78) IXREF RS(256)
 *            CODE (0x0-0x08FF)
 *            DATA (0x40)
 *            IDATA (0x80)
 *            XDATA (0x0900-0x107F)
 *            STACK (?STACK(0x90))
 *
 *------------------------------------------------------------------------------
 *
 *    INCLUDES:
 */
#include <stdlib.h>
#include "si4010.h"
#include "si4010_api_rom.h"
#include "si4010_api_BMsr.h"

/* Demo header */
#include "keyfob_demo.h"

/*------------------------------------------------------------------------------
 *
 *    FUNCTION DESCRIPTION:
 *      This function tunes the si4010 for transmission and transmits frames.
 *      It consists of two fundamental phases, setup and the transmission loop.
 *
 *      Setup Phase:
 *        The Setup Phase calls all setup function for the need API modules.
 *        These functions are to be called once per boot of the chip.
 *          vSys_Setup
 *          vPa_Setup
 *          vOds_Setup
 *          vStl_EncodeSetup
 *          vFCast_Setup
 *
 *      Transmission Loop Phase:
 *        The Transmission Loop Phase is a loop which transmits the proper frame
 *		  once a button is pushed. Frequency casting, FSK adjustment
 *        and PA tuning are all performed between each transmission.
 *
 *------------------------------------------------------------------------------
 */

void main(void)
{
  /*------------------------------------------------------------------------------
   *    SETUP PHASE
   *------------------------------------------------------------------------------
   */
  //Set DMD interrupt to high priority,
  // any other interrupts have to stay low priority
  PDMD=1;

  // Parameters Initialization.
  vAppInitial();


  // Turn LED control ON if burned to user or run mode
#if (!defined BOARD_TEST)
  if ((PROT0_CTRL & M_NVM_BLOWN) > PART_FACTORY)
#endif
    GPIO_LED = 1;

  // Call the system setup. This just for initialization.
  // Argument of 20 just configures the SYS module such that the
  // bandgap can be 800ms
  vSys_Setup( 20 );

  //Run first power up test (and function if required),
  //if part was burned to user or run mode
  if ((PROT0_CTRL & M_NVM_BLOWN) > PART_FACTORY)
  {
    if (0 != (SYSGEN & M_POWER_1ST_TIME))
    {
      P0 |= M_GPIO3 | M_GPIO2 | M_GPIO1;
      P0CON &= ~(M_GPIO3 | M_GPIO2 | M_GPIO1);
      PORT_CTRL &= ~(M_PORT_MATRIX | M_PORT_ROFF);
      PORT_CTRL |=  M_PORT_MATRIX;
      PORT_CTRL |=  M_PORT_STROBE;
      PORT_CTRL &= (~M_PORT_STROBE);

      vSys_FirstPowerUp();
    }
  }

  // Turn LED control OFF
  GPIO_LED = 0;

  // Setup the bandgap for working with the temperature sensor here
  vSys_BandGapLdo( 1 );

  // Setup the BSR
  vBMsr_Setup( &rBMsrSetup );

  // Setup the RTC to tick every 5ms and clear it. Keep it disabled.
  RTC_CTRL = (0x07 << B_RTC_DIV) | M_RTC_CLR; //RTC isr @ 5ms

  // Enable the RTC
  RTC_CTRL |= M_RTC_ENA;

  // Enable the RTC interrupt and global interrupt enable
  ERTC = 1;
  EA = 1;

  // Setup the PA
  vPa_Setup( &rPaSetup );

  // ODS setup
  vOds_Setup( &rOdsSetup );

  // Setup the STL encoding for none. No encode function therefore we
  // leave the pointer at NULL.
  vStl_EncodeSetup( bEnc_NoneNrz_c, NULL );


  // Setup frequency casting .. needed to be called once per boot
  vFCast_Setup();



  /*------------------------------------------------------------------------------
   *    TRANSMISSION LOOP PHASE
   *------------------------------------------------------------------------------
   */

  /* Application loop, including push button state analyzation and transmission. */
  while(1)
  {
    // Buttons analyzation and update task flag. 
    vButtonCheck();

    if (bAp_ButtonState)
    {
      // Set frame size and frame header pointer 
      vPacketAssemble();
      if (lPayLoad)
      {
        // Start to transmit
        vRtl_RepeatTxLoop();
      }
    }
    else if( (lSys_GetMasterTime() >> 5) > bMaxWaitForPush_c )
    {
      // For Matrix-mode during standby set GPIO[3:1] as inputs:
      // Write 1 to the port & make the driver option as open-drain
      // Shutdown if timeout, currently after about 1.6s,
      // Not used if part was not burned to user or run mode (for debugging with IDE)
      if ((PROT0_CTRL & M_NVM_BLOWN) > PART_FACTORY)
      {
        P0 |= M_GPIO3 | M_GPIO2 | M_GPIO1;
        P0CON &= ~(M_GPIO3 | M_GPIO2 | M_GPIO1);
        PORT_CTRL &= ~(M_PORT_MATRIX | M_PORT_ROFF);
        PORT_CTRL |=  M_PORT_MATRIX;
        PORT_CTRL |=  M_PORT_STROBE;
        PORT_CTRL &= (~M_PORT_STROBE);

        vSys_Shutdown();
      }
    }
  }
}


/*------------------------------------------------------------------------------
 *
 *    FUNCTION DESCRIPTION:
 *      Initialization of Application parameters. 
 *
 *------------------------------------------------------------------------------
 */
void vAppInitial(void)
{



  // Disable the Matrix mode & the Roff modes on GPIO[3:1]
  PORT_CTRL &= ~(M_PORT_MATRIX | M_PORT_ROFF | M_PORT_STROBE);
  PORT_CTRL |=  M_PORT_STROBE;
  PORT_CTRL &= (~M_PORT_STROBE);

  P0CON &= ~(M_GPIO7 | M_GPIO6 | M_GPIO4 | M_GPIO0);
  P1CON &= ~(M_GPIO9 | M_GPIO8);
  P0 	|= (M_GPIO7 | M_GPIO6 | M_GPIO4 | M_GPIO0);
  P1 	|= (M_GPIO9 | M_GPIO8);

  // Set GPIO[3:2] port output to 1 and GPIO1 to 0 --> address first row in button-matrix
  P0 	|= (M_GPIO3 | M_GPIO2);
  P0 	&= ~(M_GPIO1);

  // P0 = M_PORT_TST_IND;

  // Turn LED control off
  //  GPIO_LED = 0;
  // Set LED intensity .. acceptable values are 0 (off) or 1, 2, and 3
  vSys_LedIntensity( 3 );

  bPrevBtn = 0;
  bAp_RunState = 0;
  // Initial debounce time.
  bIsr_DebounceCount = 0;
  // debounce time definition, the unit is defined by the RTC interrupt time
  //(5ms in the demo).It means that the interval between calls to the
  // vBMsr_Service() function is 5*8=40ms.
  // BSR parameters initialization
#if (!defined BOARD_TEST)
  rBMsrSetup.bMatrixRowMask = 0x05; /* Enable some rows in button-matrix */	
#else
  rBMsrSetup.bMatrixRowMask = 0x07; /* Enable some rows in button-matrix */	
#endif

#if (!defined BOARD_TEST)
  rBMsrSetup.bMatrixColMask = 0x2A; /* Enable some columns in button-matrix */		
#else
  rBMsrSetup.bMatrixColMask = 0x3F; /* Enable some columns in button-matrix */		
#endif

  rBMsrSetup.prPtsReserveHead = arBMsrPtsPlaceHolder;
  rBMsrSetup.bPtsSize = 10;
  rBMsrSetup.bPushQualThresh = 3;

  // Setup the PA.
  // fAlpha and fBeta has to be set based on antenna configuration.
  // Chose a PA level and nominal cap. Both values come from
  // the calculation spreadsheet.
  rPaSetup.fAlpha      = 0.0;
  rPaSetup.fBeta       = 0.0;
  rPaSetup.bLevel      = 60;
  rPaSetup.wNominalCap = 256;
  rPaSetup.bMaxDrv     = 0;

  // Setup the ODS
  rOdsSetup.bModulationType = bRke_FskMod_c; // Use FSK 
  rOdsSetup.bClkDiv         = 5;
  rOdsSetup.bEdgeRate       = 0;

  // Set group width to 7, which means 8 bits/encoded byte to be transmitted.
  // The value must match the output width of the data encoding function
  // set by the vStl_EncodeSetup() below!
  rOdsSetup.bGroupWidth     = 7;
  // 24MHz / (bClkDiv+1) / 9.6kbps = 417
  rOdsSetup.wBitRate        = 417;

  // Configure the warm up intervals LC: 8, DIV: 4, PA: 4
  rOdsSetup.bLcWarmInt  = 8;
  rOdsSetup.bDivWarmInt = 4;
  rOdsSetup.bPaWarmInt  = 4;

  // Set external crystal oscillator parameters - 10MHz, 12pF
  // currently XO is not used in this example
  rFCast_XoSetup.fXoFreq = 10000000;//12729633;
  rFCast_XoSetup.bLowCap = 1;

  return;
}

/*------------------------------------------------------------------------------
 *
 *    FUNCTION DESCRIPTION:
 *      This function contains the loop which consists of three procedures,
 *      tx setup, tx loop and tx shutdown in the application.
 *      During waiting for repeat transmission, check button state.
 *      Once any new push button is detected, then transmit the new packet 
 *      instead of the current packet.
 *
 *------------------------------------------------------------------------------ 
 */
void vRtl_RepeatTxLoop(void)

{ 

  // Repeat packet 8 times for each button packet transmission.
  bRepeatCount = bRepeatCount_c;



  do
  {
    // Setup the bandgap  
    vSys_BandGapLdo( 1 );

    // Save current timestamp for repeat transmission interval checking later. 
    lTimestamp = lSys_GetMasterTime();

    // Run at 433.46 MHz (Upper frequency).Input is in [Hz]. 
    // The function also calculates necessary data to be used by the single Tx loop
    // main function, vStl_SingleTxLoop(), for fine frequency correction during TX
    vFCast_Tune( fOperatingFrequency );

    // bFskDeviation comes from the calculation spreadsheet.
    vFCast_FskAdj( bFskDeviation ); 	

    // Now wait until there is a temperature value  
    while ( 0 == bDmdTs_GetSamplesTaken() ){}

    // Tune the PA with the current temperature as an argument 
    vPa_Tune( iDmdTs_GetLatestTemp());

    // Sets up the temperature sensor and demodulator,
    // enable ODS before entering the single tx loop. 
    vStl_PreLoop();

    // Then assign the first address of the frame buffer to
    // the packet pointer used for transmission. 
    pbRke_FrameHead = abButtonFrame;

    // call the single tx loop function 
    vStl_SingleTxLoop(pbRke_FrameHead,
        bFrameSize_c);

    // Disable the temperature sensor and ODS
    vStl_PostLoop();

    // Disable Bandgap and LDO 
    vSys_BandGapLdo(0);


    //turn LED on
#if (!defined BOARD_TEST)
    if ((PROT0_CTRL & M_NVM_BLOWN) > PART_FACTORY)         //if part is burned to user or run mode.
#endif
      GPIO_LED = 1;

    // Wait repeat interval. 
    while ( (lSys_GetMasterTime() - lTimestamp) < wRepeatInterval );

    //turn LED off
    GPIO_LED = 0;
  }while(--bRepeatCount);

  // After repeat transmission, clear all tasks flag. 
  bAp_ButtonState = 0;

  // Clear time value for next button push detecting. 
  vSys_SetMasterTime(0);
  return;
}

/*
 *------------------------------------------------------------------------------
 *
 *    FUNCTION DESCRIPTION:
 *      Update the master time by 5, every time this isr is run, and call the 
 * 		Button Service Routine
 *
 */
void isr_rtc (void) interrupt INTERRUPT_RTC using 1

    {
  // clear the RTC_INT 
  RTC_CTRL &= ~M_RTC_INT;
  vSys_IncMasterTime(5); 
  bIsr_DebounceCount ++;
  if ((bIsr_DebounceCount % bDebounceInterval_c) == 0)
  {
    vBMsr_Service();
  }

  return;
    }

/*
 *------------------------------------------------------------------------------
 *
 *    FUNCTION DESCRIPTION:
 *      Update bAp_ButtonState which indicates what to be transmitted.
 *      Check the elements on PTS (push tracking strcture) to see if any GPIO
 *      has been pressed or released.
 *      If any new pressed button has detected, the corresponding flag will be set and
 *      the associated information will be transmitted in the application loop procedure.
 *
 *------------------------------------------------------------------------------ 
 */
void vButtonCheck( void)
{ 
  ERTC = 0;		// Disable RTC interrupt, otherwise button state might be corrupted.
  bAp_ButtonState = 0;

#if (defined RUN_FIFO_TEST)
  // Waits until the FIFO is full than send all packets with time delay
  if (bBMsr_GetPtsItemCnt() == rBMsrSetup.bPtsSize) // PTS reached its end start to pop all the entries
//  if (bBMsr_GetPtsItemCnt() == 4)
  	bAp_RunState = 1;

  if (bBMsr_GetPtsItemCnt() == 0) // PTS is empty
  	bAp_RunState = 0;

  if ( !(lSys_GetMasterTime() % 500) ) //Delay
  {
  	if (bAp_RunState  == 1)
	  {
		  bAp_RunState  = 2;
		  bAp_ButtonState = wBMsr_Pop() & 0xFF;
	  }
  }
  else if (bAp_RunState == 2)
	  bAp_RunState  = 1;

#else	
  if (bBMsr_GetPtsItemCnt()) // There are entries in the FIFO
  {

    bAp_ButtonState = wBMsr_Pop() & 0xFF;

    if (bPrevBtn & 0x3F) //Exclude releases from transfer, filter for columns only
    {
      bPrevBtn = bAp_ButtonState;
      bAp_ButtonState = 0;
    }
    else
    {
      bPrevBtn = bAp_ButtonState;
    }

  }
#endif

  ERTC = 1;                 // Enable RTC interrupt
  return;
}

/*------------------------------------------------------------------------------
 *
 *    FUNCTION DESCRIPTION:
 *      This function is used to update the packet for transmission,
 *      it is called after vAp_AnalyzeButton(), according to the
 *      the result of buttons analyzation to decide which packet
 *      to be transmited.
 *
 *------------------------------------------------------------------------------ 
 */
void vPacketAssemble ( void)
{
  /* Normal packet transmission mode, send the associated packet
   * according to different button pressed.
   * For the demo application in conjunction with si4320 receiver,
   * there are 18 different packets corresponding to the button-matrix.
   * First 15 buttons send packet with the binary code of their number, 
   * LEDs of the receiver will flash.
   * Button 16, 17 & 18 send packets with binary code of 0010, 0100 & 1000, 
   * LEDs of the receiver will be ON.
   * Note that Output Data Serializer (ODS) is in little endian fashion,
   * bit 0 first and inverted. So each byte of the packet data should be
   * converted properly before writing into frame buffer.
   */

  switch (bAp_ButtonState)
  {
    case (bMatrixCol1_c + bMatrixRow1_c):
	          lPayLoad = lFramePayLoad01_c;
    break;
    case (bMatrixCol1_c + bMatrixRow2_c):
	          lPayLoad = lFramePayLoad02_c;
    break;
    case (bMatrixCol1_c + bMatrixRow3_c):
	          lPayLoad = lFramePayLoad03_c;
    break;
    case (bMatrixCol2_c + bMatrixRow1_c):
	          lPayLoad = lFramePayLoad04_c;
    break;
    case (bMatrixCol2_c + bMatrixRow2_c):
	          lPayLoad = lFramePayLoad05_c;
    break;
    case (bMatrixCol2_c + bMatrixRow3_c):
	          lPayLoad = lFramePayLoad06_c;
    break;
    case (bMatrixCol3_c + bMatrixRow1_c):
	          lPayLoad = lFramePayLoad07_c;
    break;
    case (bMatrixCol3_c + bMatrixRow2_c):
	          lPayLoad = lFramePayLoad08_c;
    break;
    case (bMatrixCol3_c + bMatrixRow3_c):
	          lPayLoad = lFramePayLoad09_c;
    break;
    case (bMatrixCol4_c + bMatrixRow1_c):
	          lPayLoad = lFramePayLoad10_c;
    break;
    case (bMatrixCol4_c + bMatrixRow2_c):
	          lPayLoad = lFramePayLoad11_c;
    break;
    case (bMatrixCol4_c + bMatrixRow3_c):
	          lPayLoad = lFramePayLoad12_c;
    break;
    case (bMatrixCol5_c + bMatrixRow1_c):
	          lPayLoad = lFramePayLoad13_c;
    break;
    case (bMatrixCol5_c + bMatrixRow2_c):
	          lPayLoad = lFramePayLoad14_c;
    break;
    case (bMatrixCol5_c + bMatrixRow3_c):
	          lPayLoad = lFramePayLoad15_c;
    break;
    case (bMatrixCol6_c + bMatrixRow1_c):
	          lPayLoad = lFramePayLoad16_c;
    break;
    case (bMatrixCol6_c + bMatrixRow2_c):
	          lPayLoad = lFramePayLoad17_c;
    break;
    case (bMatrixCol6_c + bMatrixRow3_c):
	          lPayLoad = lFramePayLoad18_c;
    break;
    default:
      lPayLoad = 0;
      break;
  }

  // Fill frame buffer with the corresponding button data.
  abButtonFrame[0] = (lFramePreamble_c >> 16) & 0xFF;
  abButtonFrame[1] = (lFramePreamble_c >> 8) & 0xFF;
  abButtonFrame[2] = lFramePreamble_c & 0xFF;
  abButtonFrame[3] = (wFrameSyncWord_c >> 8) & 0xFF;
  abButtonFrame[4] = wFrameSyncWord_c & 0xFF;
  abButtonFrame[5] = ((BYTE *) &lPayLoad)[1];
  abButtonFrame[6] = ((BYTE *) &lPayLoad)[2];
  abButtonFrame[7] = ((BYTE *) &lPayLoad)[3];

  return;
}



