/*------------------------------------------------------------------------------
 *                          Silicon Laboratories, Inc.
 *                           http://www.silabs.com
 *                               Copyright 2010
 *------------------------------------------------------------------------------
 *
 *    FILE:       fcast_demo_main.c
 *    TARGET:     Si4010
 *    TOOLCHAIN:  Generic
 *    DATE:       May 05, 2010, Wednesday
 *    RELEASE:    2.1 (JimG), ROM version 02.00, trim 3
 *
 *------------------------------------------------------------------------------
 *
 *    DESCRIPTION:
 *      This file contains the main function for the Keil toolchain
 *      sample fcast_demo project.
 *
 *      BL51 linker directives for building this application:
 *
 *      BL51: PL(68) PW(78) IXREF RS(256)
 *            CODE (0x0-0x03FF)
 *            XDATA (0x0E00-0x107F)
 *            STACK (?STACK(0x90))
 *
 *------------------------------------------------------------------------------
 *
 *    INCLUDES:
 */
#include <stdlib.h>
#include "si4010.h"
#include "si4010_api_rom.h"

// Demo header 
#include "fcast_demo.h"
/*------------------------------------------------------------------------------
 *
 *    FUNCTION DESCRIPTION:
 *      This function tunes the si4010 for transmission and transmits frames.
 *      It consists of two fundamental phases, setup and the transmission loop.
 *
 *      Setup Phase:
 *        The Setup Phase calls all setup function for the need API modules.
 *        These functions are to be called once per boot of the chip.
 *          vSys_Setup
 *          vPa_Setup
 *          vOds_Setup
 *          vStl_EncodeSetup
 *          vFCast_Setup
 *
 *      Transmission Loop Phase:
 *        The Transmission Loop Phase is a loop which transmits a frame once
 *        any button is pushed.  Frames will continue to be transmitted as
 *        long as the button is held down.  Frequency casting, FSK adjustment
 *        and PA tuning are all performed between each transmission.
 *
 *------------------------------------------------------------------------------
 */

void    main(void)
{

//Set DMD interrupt to high priority,
// any other interrupts have to stay low priority
  PDMD=1;
// Disable the Matrix and Roff modes on GPIO[3:1] 
  PORT_CTRL &= ~(M_PORT_MATRIX | M_PORT_ROFF | M_PORT_STROBE);
  PORT_CTRL |=  M_PORT_STROBE;
  PORT_CTRL &= (~M_PORT_STROBE);

// Turn LED control off 
  GPIO_LED = 0;

//------------------------------------------------------------------------------
//    SETUP PHASE
//------------------------------------------------------------------------------


// Call the system setup. This just for initialization.
// Argument of 1 just configures the SYS module such that the
// bandgap can be turned off if needed. 
  vSys_Setup( 1 );

// Setup the bandgap for working with the temperature sensor here 
  vSys_BandGapLdo( 1 );

// Setup the PA.
// fAlpha and fBeta has to be set based on antenna configuration.
// Chose a PA level and nominal cap. Both values come from
// the calculation spreadsheet.
  rPaSetup.fAlpha      = 0.0;
  rPaSetup.fBeta       = 0.0;
  rPaSetup.bLevel      = 60;
  rPaSetup.wNominalCap = 256;
  rPaSetup.bMaxDrv     = 0;

// Setup the PA 
  vPa_Setup( &rPaSetup );

 // Setup the ODS 
  rOdsSetup.bModulationType = 1; // Use FSK 
  rOdsSetup.bClkDiv         = 5;
  rOdsSetup.bEdgeRate       = 0;

// Set group width to 7, which means 8 bits/encoded byte to be transmitted.
// The value must match the output width of the data encoding function
// set by the vStl_EncodeSetup() below! 
  rOdsSetup.bGroupWidth     = 7;
  rOdsSetup.wBitRate        = 417;

// Configure the warm up intervals LC: 8, DIV: 4, PA: 4 
  rOdsSetup.bLcWarmInt  = 8;
  rOdsSetup.bDivWarmInt = 4;
  rOdsSetup.bPaWarmInt  = 4;

// ODS setup 
  vOds_Setup( &rOdsSetup );

// Setup the STL encoding for none. No encode function therefore we
// leave the pointer at NULL.  
  vStl_EncodeSetup( bEnc_NoneNrz_c, NULL );

// Generate the frame to use. Example array is fixed in CODE
// as constant for this example. Assign the beginning of the array
// to the pbFrameHead pointer.  
  pbFrameHead = (BYTE xdata *) abFCastDemo_FrameArray;

// Setup frequency casting .. needed to be called once per boot 
  vFCast_Setup();

// Read the battery voltage in mV with loading .. 0. This is just
// for demonstration purposes only and the read value is not used.
// If the bBatteryWait = 0 then unloaded measurement is taken.  
  bBatteryWait = 0;
  iBatteryMv = iMVdd_Measure( bBatteryWait );

// Make loaded measurement .. 200 * 17 = 3.4ms wait after loading 
  bBatteryWait = 200;
  iBatteryMv = iMVdd_Measure( bBatteryWait );

//------------------------------------------------------------------------------
//    TRANSMISSION LOOP PHASE
//------------------------------------------------------------------------------


// Infinite loop here to continue tuning and transmitting a frame.
// If only 1 frame is desired per button push this while loop could be
// eliminated and the vStl_PostLoop() could be followed by a call to
// vSys_Shutdown()  
  while ( 1 )
  {

//  Wait for a button press .. wait here in infinite loop for a button
//  to be pushed or held. If not buttons are pushed then wait for button
//  push. See the comment about vSys_Shutdown() above.
//  Very simple button monitor, no deglitching. In real application
//  the user should/could use the RTC interrupt, master time,
//  and vBsr_Service() routines to deglitch and monitor input buttons.  
    while ( (0xdf == (P0 & 0xdf)) && (0x03 == (P1 & 0x03)) ) {}

//  Run at 433.46 MHz.  Continue to tune frequency between frames.
//  This is not absolutely necessary as the Single transmission loop
//  also adjusts for temperature drift. It calculate values used by
//  the vStl_SingleTxLoop() during transmission. Input is in [Hz].  
    vFCast_Tune( 433460000.0 );

//  Call vFCast_FskAdj with the maximum FSK adjustment.
//  Input parameter value comes from the calculation spreadsheet.  
    vFCast_FskAdj( 127 );

//  Now wait until there is a demodulated temperature sample  
    while ( 0 == bDmdTs_GetSamplesTaken() ){}

//  Tune the PA with the current temperature as an argument 
    vPa_Tune( iDmdTs_GetLatestTemp() );

//  -- Run the Single Tx Loop 

//  Prepare for Tx 
    vStl_PreLoop();

//  Turn the LC, DIV, and PA on and send data 
    vStl_SingleTxLoop( pbFrameHead,
                       bFCastDemo_MaxFrameSize_c );

//  Post processing, Tx end, turn LC, DIV, and PA off 
    vStl_PostLoop();
  }
}

//------------------------------------------------------------------------------

