/*------------------------------------------------------------------------------
 *                          Silicon Laboratories, Inc.
 *                           http://www.silabs.com
 *                               Copyright 2010
 *------------------------------------------------------------------------------
 *
 *    FILE:       fstep_demo_run_main.c
 *    TARGET:     Si4010
 *    TOOLCHAIN:  Keil
 *    DATE:       May 18, 2010, Tuesday
 *    RELEASE:    2.0 (JimG, MPech), ROM version 02.00, trim 3
 *
 *------------------------------------------------------------------------------
 *
 *    DESCRIPTION:
 *      This file contains the main function for the Keil toolchain
 *      sample fstep_demo_run target.
 *
 *      BL51 linker directives for building this application:
 *
 *      BL51: PL(68) PW(78) IXREF RS(256)
 *            CODE (0x0-0X0EFF)
 *            XDATA (0X0F00-0X107F)
 *            STACK (?STACK(0x90))
 *
 *------------------------------------------------------------------------------
 *
 *    INCLUDES:
 */
#include <stdlib.h>
#include "si4010.h"
#include "si4010_api_rom.h"

// For use si4010_api_add.lib library the following header is required 
#include "si4010_api_add.h"

// Demo header 
#include "fstep_demo.h"

/*------------------------------------------------------------------------------
 *
 *    FUNCTION DESCRIPTION:
 *      The function uses the FStep module to tune the si4010 for transmission
 *      and transmits frames. It is dependent on the fstep_demo_setup
 *      program being run before this program is downloaded to the part.
 *
 *      The program fstep_demo_setup and this program share the byte
 *      array abFStepDemo_Data. For both programs to share the same
 *      array data value the array must be put in a fixed address
 *      in an address space which will not be overwritten when the
 *      run part of the program, fstep_demo_run, is loaded
 *      to the device.
 *
 *      For this demo the 15 byte array abFStepDemo_Data[15] is located
 *      in XDATA at the address 0x10d8. See the fstep_demo.h for the
 *      array location. The user is free to put the array to DATA
 *      or IDATA, but the fixed address location _at_ specifier
 *      must be user in all scenarios.
 *
 *      This run function is made up of two fundamental phases, setup and
 *      the transmission loop.
 *
 *      Setup Phase
 *      ~~~~~~~~~~~
 *      The setup phase calls all setup functions for the needed API modules.
 *      These functions are to be called once per boot of the chip.
 *
 *        vSys_Setup
 *        vPa_Setup
 *        vOds_Setup
 *        vFCast_Setup
 *        vFStep_Apply <-- applies chip setting acquired in
 *                         fstep_demo_setup as output of the
 *                         vFCast_Tune() call followed by bFStep_Collect()
 *
 *      Transmission Loop Phase
 *      ~~~~~~~~~~~~~~~~~~~~~~~
 *      The transmission loop phase is a loop which transmits a frame once
 *      any button is pushed. Frames will continue to be transmitted as
 *      long as the button is held down. FSK adjustment and PA tuning are
 *      performed between each transmission.
 *
 *------------------------------------------------------------------------------
 */
void    main(void)
{

//------------------------------------------------------------------------------
//    VARIABLES:


// Structure for setting up the ODS .. must be in XDATA 
  tOds_Setup xdata rOdsSetup;

// Structure for setting up the PA .. must be in XDATA 
  tPa_Setup xdata  rPaSetup;

// Pointer to frame head 
  BYTE xdata    *pbFrameHead;

// Other variables 
  BYTE          bFreqInd;       // Saved frequency index 

//------------------------------------------------------------------------------
 

// Disable the Matrix and Roff modes on GPIO[3:1] 
  PORT_CTRL &= ~(M_PORT_MATRIX | M_PORT_ROFF | M_PORT_STROBE);
  PORT_CTRL |=  M_PORT_STROBE;
  PORT_CTRL &= (~M_PORT_STROBE);

//------------------------------------------------------------------------------
//    SETUP PHASE
//------------------------------------------------------------------------------
 
//Set DMD interrupt to high priority,
// any other interrupts have to stay low priority
  PDMD=1;
// Call the system setup. This just for initialization.
// Argument of 1 just configures the SYS module such that the
// bandgap can be turned off if needed.  
  vSys_Setup( 1 );

// Setup the bandgap for working with the temperature sensor here.
// bSys_FirstBatteryInsertWaitTime set to non zero value. 
  vSys_BandGapLdo( 1 );

// Setup the PA.
// Zero out Alpha and Beta here. They have to do with the antenna.
// Chose a nice high PA Level. This value, along with the nominal cap
// come from the CAL SPREADSHEET 
  rPaSetup.fAlpha      = 0;
  rPaSetup.fBeta       = 0;
  rPaSetup.bLevel      = 60;
  rPaSetup.wNominalCap = 256;
  rPaSetup.bMaxDrv     = 0;

// Required bPA_TRIM clearing before calling vPa_Setup() 
  vPa_Setup( &rPaSetup );

 // Setup the ODS 
  rOdsSetup.bModulationType = 1;  // Use FSK 
  rOdsSetup.bClkDiv         = 5;
  rOdsSetup.bEdgeRate       = 0;

// Set group width to 7, which means 8 bits/encoded byte to be transmitted.
// The value must match the output width of the data encoding function
// set by the vStl_EncodeSetup() below! 
  rOdsSetup.bGroupWidth     = 7;
  rOdsSetup.wBitRate        = 417;

// Configure the warm up intervals LC: 8, DIV: 4, PA: 4 
  rOdsSetup.bLcWarmInt  = 8;
  rOdsSetup.bDivWarmInt = 4;
  rOdsSetup.bPaWarmInt  = 4;

// ODS setup 
  vOds_Setup( &rOdsSetup );

// Setup the STL encoding for none. No encode function therefore we
// leave the pointer at NULL. 
  vStl_EncodeSetup( bEnc_NoneNrz_c, NULL );

// Generate the frame to use. Example array is fixed in CODE
// as constant for this example. Assign the beginning of the array
// to the pbFrameHead pointer. 
  pbFrameHead = (BYTE xdata *) abFStepDemo_FrameArray;

// Setup and run the frequency casting. 
  vFCast_Setup();

// Collect items for 3 frequencies in advance 

// Freq_0 
  vFCast_Tune( 433.46e6 );
  bFStep_Collect( (BYTE xdata *) &(arFStepDemo_Data[0]) );

// Freq_1 
  vFCast_Tune( 430.0e6 );
  bFStep_Collect( (BYTE xdata *) &(arFStepDemo_Data[1]) );

// Freq_2 
  vFCast_Tune( 440.0e6 );
  bFStep_Collect( (BYTE xdata *) &(arFStepDemo_Data[2]) );

//------------------------------------------------------------------------------
//    TRANSMISSION LOOP PHASE
//------------------------------------------------------------------------------
 

// Infinite loop here to continue tuning and transmitting a frame.
// only 1 frame is desired per button push this while loop could be
// eliminated and the vStl_PostLoop() could be followed by a call to
// vSys_Shutdown() 

  while ( 1 )
  {

//  Wait for a button press .. wait here in infinite loop for a button
//  to be pushed or held. If not buttons are pushed then wait for button
//  push. See the comment about vSys_Shutdown() above. 
    while ( (0xdf == (P0 & 0xdf)) && (0x03 == (P1 & 0x03)) ) {}

//  Decide what frequency to use based on the input buttons 
    bFreqInd = 0;
    if ( 0 == GPIO1 )
    {
       bFreqInd = 1;
    }
    else
    {
      if ( 0 == GPIO2 )
      {
        bFreqInd = 2;
      }
    }

/*  The DmdTs module must be enabled for temperature sensing here
 *  because it is disabled in the vStl_PostLoop call. This can be
 *  removed if one wishes to tune the PA outside this loop
 *  or if tuning by calling vFStep_Apply() here.
 *
 *  If the vFStep_Apply() is used it calls this function internally.
 *  But be aware that vFStep_Apply() behaves the same way as vFCast_Tune(),
 *  so it calls
 *    vDmdTs_RunForTemp( 3 );
 *    vSys_ForceLc();
 *  internally to enable the DMD TS and forces LC to be turned on.
 *  In that case the vDmdTs_RunForTemp( 3 ) call below
 *  can be removed.
 *
 *  If the user is using the vFStep_ApplyFast(), neither the DMD TS
 *  nor the LC is turned on and the following line is required.
 *
 *  Left in the code even if it is not needed for vFStep_Apply().
 */
    vDmdTs_RunForTemp( 3 );  // Skip 3 samples. 

//  Tune to the saved frequency 
    vFStep_Apply( (BYTE xdata *) &(arFStepDemo_Data[bFreqInd]) );

//  Call vFCast_FskAdj with the maximum FSK adjustment.
//  127 from the CAL SPREADSHEET 
    vFCast_FskAdj( 127 );

//  Wait until there is a demodulated temperature sample  
    while ( 0 == bDmdTs_GetSamplesTaken() ) {}

//  Tune the PA with the current temperature as an argument 
    vPa_Tune( iDmdTs_GetLatestTemp() );

//  -- Run the Single Tx Loop 

//  Prepare for Tx 
    vStl_PreLoop();

//  Turn the LC, DIV, and PA on and send data 
    vStl_SingleTxLoop( pbFrameHead,
                       bFStepDemo_MaxFrameSize_c );

//  Post loop processing 
    vStl_PostLoop();
  }
}

//------------------------------------------------------------------------------
 
