/*------------------------------------------------------------------------------
 *                          Silicon Laboratories, Inc.
 *                           http://www.silabs.com
 *                               Copyright 2010
 *------------------------------------------------------------------------------
 *
 *    FILE:       OD_main.c
 *    TARGET:     Si4010
 *    TOOLCHAIN:  Keil
 *    RELEASE:    1.1 ; ROM version 02.00
 *    AUTHOR:     Miklos Lukacs
 *    DATE:       May 31, 2010
 *
 *------------------------------------------------------------------------------
 *
 *    DESCRIPTION:
 *      This file contains the main module of the Overlay demo.
 *      Handles the button press processing and calls the overlay modules:
 *       - overlay 1: RF packet construction
 *       - overlay 2: RF packet TX
 *
 *      BL51 linker directives for building this application:
 *
 *            PL(68) PW(78) IXREF RS(256)
 *            CODE (0x0-0x02FF)
 *            DATA (0x40)
 *            IDATA (0x80)
 *            XDATA (0x0300-0x03FF)
 *            STACK (?STACK(0xA0))
 */


/*------------------------------------------------------------------------------
 *
 *    INCLUDES:
 */
#include <stdlib.h>
#include "si4010.h"
#include "si4010_api_rom.h"

#include "OD_main.h"
#include "OD_global.h"



/*------------------------------------------------------------------------------
 *
 *    FUNCTION DESCRIPTION:
 *      This is the main() function in the Main Module of the Overlay Demo.
 *      It relaizes the Keyfob demo functionality while demonstrating the 
 *      Memory overlay technique.
 *      The project is separated into 3 Modules:
 *          - Main Module:      - Initializes the application 
 *                              - handles Memory Overlay
 *                              - processes the Buttons presses
 *          - Overlay Module 1: - assembles the RF packet
 *          - Overlay Module 2: - transmits the RF packet.
 *        
 *      The Main Module consists of two fundamental phases:
 *          - setup
 *          - main application loop.
 *
 *      Setup Phase:
 *        The Setup Phase calls all setup function for the need API modules.
 *        These functions are to be called once per boot of the chip.
 *          - vSys_Setup
 *          - vPa_Setup
 *          - vOds_Setup
 *          - vStl_EncodeSetup
 *          - vFCast_Setup
 *
 *      Main Application Loop:
 * 			- push button state analyzation
 *          - copy to RAM and run Overlay Module 1 to assemble the RF packet
 * 		    - copy to RAM and run Overlay Module 2 to transmit the RF packet. 
 *
 *------------------------------------------------------------------------------
 */

void main(void)
{


/*------------------------------------------------------------------------------
 *    SETUP PHASE
 *------------------------------------------------------------------------------
 */
//Set DMD interrupt to high priority,
// any other interrupts have to stay low priority
  PDMD=1;

// Parameters Initialization. 
  vAppInitial();

// Set pointer to overlay function. Note that vOvrl_FuncReserve is linked to 0x0500 
  pOvrl_Func = vOvrl_FuncReserve;

// Call the system setup. This just for initialization.
// Argument of 1 just configures the SYS module such that the
// bandgap can be turned off if needed.
  vSys_Setup( 1 );

// Setup the bandgap for working with the temperature sensor here 
  vSys_BandGapLdo( 1 );
 
// Setup the BSR 
  vBsr_Setup( &rBsrSetup );

// Setup the RTC to tick every 5ms and clear it. Keep it disabled. 
  RTC_CTRL = (0x07 << B_RTC_DIV) | M_RTC_CLR;

// Enable the RTC 
  RTC_CTRL |= M_RTC_ENA;

// Enable the RTC interrupt and global interrupt enable 
  ERTC = 1;
  EA = 1;

// Setup the PA 
  vPa_Setup( &rPaSetup );

// ODS setup 
  vOds_Setup( &rOdsSetup );

// Setup the STL encoding for none. No encode function therefore we
// leave the pointer at NULL. 
  vStl_EncodeSetup( bEnc_NoneNrz_c, NULL );


// Setup frequency casting .. needed to be called once per boot 
  vFCast_Setup();

// Read the battery voltage in mV with loading .. 0. This is just
// for demonstration purposes only and the read value is not used.
// If the bBatteryWait = 0 then unloaded measurement is taken. 
  bBatteryWait = 0;
  iBatteryMv = iMVdd_Measure( bBatteryWait );

// Make loaded measurement .. 200 * 17 = 3.4ms wait after loading 
  bBatteryWait = 200;
  iBatteryMv = iMVdd_Measure( bBatteryWait );

/*------------------------------------------------------------------------------
 *    MAIN APPLICATION LOOP
 *------------------------------------------------------------------------------
 */

  while(1)
  {
    // Buttons analyzation and update task flag. 
    vButtonCheck();

    if (bAp_ButtonState)
    {
	  // Load and call overlay function from NVM 
	  wNvmAddrOvrl = wNvmAddr_PacketAssembler;
	  if ( vOvrl_NvmLoad(wNvmAddrOvrl) != 0xFF )
	  {
       // Set frame size and frame header pointer 
	   // vPacketAssemble() is in the Overlay module
        #ifdef AssemblyCall
	       vOvrl_FuncCaller();		// Call overlay function from assembly 
        #endif

        #ifdef FuncPointerCall
		   (pOvrl_Func)();		   	// Call overlay function through function pointer 
        #endif 
      }
	  else
	  {	//if there was an error during NVM load
	    vSys_LedIntensity(1);
	    while(1);
	  }
	  if (lPayLoad)
	  {
	      // Start to transmit. 
		  // Load and call overlay function from NVM 
		  wNvmAddrOvrl = wNvmAddr_SendPacket;
		  if ( vOvrl_NvmLoad(wNvmAddrOvrl) != 0xFF )
		  {
		   // vRtl_RepeatTxLoop() is in the Overlay module
	        #ifdef AssemblyCall
		       vOvrl_FuncCaller();		// Call overlay function from assembly 
	        #endif

	        #ifdef FuncPointerCall
			   (pOvrl_Func)();		   	// Call overlay function through function pointer 
	        #endif 
	      }
		  else
		  {	//if there was an error during NVM load
		    vSys_LedIntensity(1);
		    while(1);
		  }
      }
    }
    else if( (lSys_GetMasterTime() >> 5) > bMaxWaitForPush )
    {
      // Shutdown if timeout, currently after about 1.6s,
      // For debugging with IDE, comment it out since it does not support boot.
//      vSys_Shutdown();
    }
  }
}


/*
 *------------------------------------------------------------------------------
 *
 *    FUNCTION DESCRIPTION:
 *      Update the master time by 5, every time this isr is run, and call the 
 * 		Button Service Routine
 *
 */
void isr_rtc (void) interrupt INTERRUPT_RTC using 2

{ 
  RTC_CTRL &= ~M_RTC_INT;	// clear the RTC_INT flag
  vSys_IncMasterTime(5); 
  bIsr_DebounceCount ++;
  if ((bIsr_DebounceCount % bDebounceInterval) == 0)
  {
    vBsr_Service();
  }
  return;
}


/*
 *------------------------------------------------------------------------------
 *
 *    FUNCTION DESCRIPTION:
 *      Update bAp_ButtonState which indicates what to be transmitted.
 *      Check the elements on PTS (push tracking strcture) to see if any GPIO
 *      has been pressed or released.
 *      If any new pressed button has detected, the corresponding flag will be set and
 *      the associated information will be transmitted in the application loop procedure.
 *
 *------------------------------------------------------------------------------ 
 */
void vButtonCheck( void)
{ 
  ERTC = 0;		// Disable RTC interrupt, otherwise button state might be corrupted.
  bAp_ButtonState = 0;
  if (bBsr_GetPtsItemCnt())
  {							// GPIO status have been changed.
							// Some buttons may have been pressed or released. 
	bAp_ButtonState = wBsr_Pop() & 0xFF;
	if (bPrevBtn)
	{
	bPrevBtn = bAp_ButtonState;
	bAp_ButtonState = 0;
	}
	else
	{
	bPrevBtn = bAp_ButtonState;
	}
  }
  ERTC = 1;                 // Enable RTC interrupt
  return;
}


/*------------------------------------------------------------------------------
 *
 *    FUNCTION DESCRIPTION:
 *      Initialization of Application parameters. 
 *
 *------------------------------------------------------------------------------
 */
 void vAppInitial(void)
{
// Disable the Matrix and Roff modes on GPIO[3:1] 
  PORT_CTRL &= ~(M_PORT_MATRIX | M_PORT_ROFF | M_PORT_STROBE);
  PORT_CTRL |=  M_PORT_STROBE;
  PORT_CTRL &= (~M_PORT_STROBE);

// Turn LED control off 
  GPIO_LED = 0;
// Set LED intensity .. acceptable values are 0 (off) or 1, 2, and 3 
  vSys_LedIntensity( 3 );

  // Initial debouce time. 
  bIsr_DebounceCount = 0;
  // debounce time definition, the unit is defined by the RTC interrupt time
  //(5ms in the demo).It means that the interval between calls to the
  // vBsr_Service() function is 5*8=40ms.
  // BSR parameters initialization 
  rBsrSetup.bButtonMask = 0x1F;	// GPIO4 ~ GPIO0 
  rBsrSetup.pbPtsReserveHead = abBsrPtsPlaceHolder;
  rBsrSetup.bPtsSize = 10;
  rBsrSetup.bPushQualThresh = 3;

// Setup the PA.
// fAlpha and fBeta has to be set based on antenna configuration.
// Chose a PA level and nominal cap. Both values come from
// the calculation spreadsheet. 
  rPaSetup.fAlpha      = 0.0;
  rPaSetup.fBeta       = 0.0;
  rPaSetup.bLevel      = 60;
  rPaSetup.wNominalCap = 256;
  rPaSetup.bMaxDrv     = 0;

 // Setup the ODS 
  rOdsSetup.bModulationType = bRke_FskMod_c; // Use FSK 
  rOdsSetup.bClkDiv         = 5;
  rOdsSetup.bEdgeRate       = 0;

// Set group width to 7, which means 8 bits/encoded byte to be transmitted.
// The value must match the output width of the data encoding function
// set by the vStl_EncodeSetup() below! 
  rOdsSetup.bGroupWidth     = 7;
// 24MHz / (bClkDiv+1) / 9.6kbps = 417 
  rOdsSetup.wBitRate        = 417;

// Configure the warm up intervals LC: 8, DIV: 4, PA: 4 
  rOdsSetup.bLcWarmInt  = 8;
  rOdsSetup.bDivWarmInt = 4;
  rOdsSetup.bPaWarmInt  = 4;

// Set external crystal oscillator parameters - 10MHz, 12pF 
// currently XO is not used in this example
  rFCast_XoSetup.fXoFreq = 10000000;//12729633;
  rFCast_XoSetup.bLowCap = 1;

  return;
}


/*------------------------------------------------------------- 
 * 
 *    FUNCTION DESCRIPTION: 
 *      Load one overlay function from NVM. 
 * 
 *------------------------------------------------------------- 
 */ 
BYTE vOvrl_NvmLoad( WORD  wiNvmAddr )

{ 
  BYTE  bStatus;       
 
  vNvm_McEnableRead();          // Enable analog hardware and NVM for read  
  vNvm_SetAddr( wiNvmAddr );    // Set the NVM block first address  
  bStatus = bNvm_CopyBlock();   // Copy the NVM block from NVM  
  vNvm_McDisableRead();         // Disable analog hardware and NVM  
 
  return ( bStatus );           // Return the status of bNvm_CopyBlock        
} 

/*------------------------------------------------------------- 
 * 
 *    FUNCTION DESCRIPTION: 
 *      Dummy definition for the overlay function, to enable  
 * 		the linker setting up the proper call tree
 * 
 *------------------------------------------------------------- 
 */ 
 void vOvrl_FuncReserve(void)
{
   EA = EA;
   return;
}
