/*------------------------------------------------------------------------------
 *                          Silicon Laboratories, Inc.
 *                           http://www.silabs.com
 *                               Copyright 2010
 *------------------------------------------------------------------------------
 *
 *    FILE:       tone_demo_main.c
 *    TARGET:     Si4010
 *    TOOLCHAIN:  Generic
 *    DATE:       May 05, 2010, Wednesday
 *    RELEASE:    2.1 (MPech), ROM version 02.00, trim 3
 *
 *------------------------------------------------------------------------------
 *
 *    DESCRIPTION:
 *      This file contains the main function for the tone demo.
 *      It MUST be run on properly trim part!
 *
 *      Then it tracks frequency with temperature by fine tuning
 *      process by repeatedly calling the vStl_SingleTxLoop() function.
 *      Ultimately, if the temperature is going to drift too much
 *      from what it was during the main tuning vFCast_Tune()
 *      call then the fine tuning process will not be able
 *      to properly track the output frequency with temperature.
 *
 *      For an example how to interrupt to CW tone output after
 *      predefined time, retune by vFCast_Tune(), and start transmitting
 *      tone again see the tone_demo_ptune_main.c for periodic tuning.
 *
 *------------------------------------------------------------------------------
 *
 *    INCLUDES:
 */
#include <stdlib.h>
#include "si4010.h"
#include "si4010_api_rom.h"

#include "tone_demo.h"


/*------------------------------------------------------------------------------
 *
 *    FUNCTION DESCRIPTION:
 *      Main function for the example.
 *
 *------------------------------------------------------------------------------
 */

void    main(void)
{

//Set DMD interrupt to high priority,
// any other interrupts have to stay low priority
  PDMD=1;
// Disable the Matrix and Roff modes 
  PORT_CTRL &= ~(M_PORT_MATRIX | M_PORT_ROFF | M_PORT_STROBE);
  PORT_CTRL |=  M_PORT_STROBE;
  PORT_CTRL &= ~M_PORT_STROBE;

// LED control bit off 
  GPIO_LED = 0;

// Call the system setup. This just for initialization.
// Argument of 1 just configures the SYS module such that the
// bandgap can be turned off if needed.  
  vSys_Setup( 1 );

// Setup the bandgap for working with the temperature sensor here.
// bSys_FirstBatteryInsertWaitTime set to non zero value. 
  vSys_BandGapLdo( 1 );

// PA setup.
// fAlpha and fBeta has to be set based on antenna configuration.
// Chose a PA level and nominal cap. Both values come from
// the calculation spreadsheet. 
  rPaSetup.fAlpha      = 0.0;
  rPaSetup.fBeta       = 0.0;
  rPaSetup.bLevel      = 60;
  rPaSetup.wNominalCap = 256;
  rPaSetup.bMaxDrv     = 0;

// PA setup 
  vPa_Setup( &rPaSetup );

// ODS setup .. results from calculation spreadsheet.
// Set to 1kbps OOK 
  rOdsSetup.bModulationType = 0; // 1 .. FSK, 0 .. OOK 
  rOdsSetup.bClkDiv         = 5;
  rOdsSetup.bEdgeRate       = 0;

// Set group width to 7, which means 8 bits/encoded byte to be transmitted.
// The value must match the output width of the data encoding function
// set by the vStl_EncodeSetup() below. 
  rOdsSetup.bGroupWidth     = 7;
  rOdsSetup.wBitRate        = 4000;  // 1kbps 

// Configure the warm up intervals LC: 8, DIV: 5, PA: 4 
  rOdsSetup.bLcWarmInt  = 8;
  rOdsSetup.bDivWarmInt = 5;
  rOdsSetup.bPaWarmInt  = 4;

// ODS setup 
  vOds_Setup( &rOdsSetup );

// Setup the STL encoding for none. No encode function therefore we
// leave the pointer at NULL. 
  vStl_EncodeSetup( bEnc_NoneNrz_c, NULL );

// Generate the frame to use. Example array is fixed in CODE
// as constant for this example. Assign the beginning of the array
// to the pbFrameHead pointer. 
  pbFrameHead = (BYTE xdata *) abTone_FrameArray;

// Setup frequency casting .. needed to be called once per boot 
  vFCast_Setup();

// Run at 433.46 MHz.  Continue to tune frequency between frames.
// This is not absolutely necessary as the Single transmission loop
// also adjusts for temperature drift. It calculate values used by
// the vStl_SingleTxLoop() during transmission. Input is in [Hz]. 
  vFCast_Tune( 433460000.0 );

// Call vFCast_FskAdj with the maximum FSK adjustment.
// Input parameter value comes from the calculation spreadsheet.
// We are not running FSK for tone. 

//   vFCast_FskAdj( 127 );

// Now wait until there is a demodulated temperature sample  
  while ( 0 == bDmdTs_GetSamplesTaken() ){}

// Tune the PA with the current temperature as an argument 
  vPa_Tune( iDmdTs_GetLatestTemp() );

// -- Run the Single Tx Loop 
  vStl_PreLoop( );

// Encoding is none .. 0. Force first byte to turn the PA on.
// Set the all 1's output if there is no data .. value 3 below. 
  ODS_CTRL = (ODS_CTRL & (~M_ODS_SHIFT_CTRL)) | (3 << B_ODS_SHIFT_CTRL);

// Keep sending the all 1's data. The constant data byte pointed
// at by the pbFrameHead must be 0xff for all ones to keep the PA
// activated in OOK. At least 1 byte must be supplied since only
// when there is some data to be transmitted then the fine tuning
// is invoked.
// The vStl_SingleTxLoop() must be called repeatedly in the loop,
// since it internally invokes fine frequency tuning.

  while ( 1 )
  {
    vStl_SingleTxLoop( pbFrameHead,
                       2 );  // Frame length in bytes 
  }

// Infinite termination loop 
  while ( 1 )
  {
    PCON |= M_IDLE;
    PCON = PCON;   // Must be used after going to IDLE 
  }

// This will never happen. 
  return;
}

//------------------------------------------------------------------------------


