/*------------------------------------------------------------------------------
 *                          Silicon Laboratories, Inc.
 *                           http://www.silabs.com
 *                               Copyright 2010
 *------------------------------------------------------------------------------
 *
 *    FILE:       tone_demo_ptune_main.c
 *    TARGET:     Si4010
 *    TOOLCHAIN:  Generic
 *    DATE:       May 05, 2010, Wednesday
 *    RELEASE:    2.1 (MPech), ROM version 02.00, trim 3
 *
 *------------------------------------------------------------------------------
 *
 *    DESCRIPTION:
 *      This file contains the main function for the tone demo.
 *      It MUST be run on properly trim part!
 *
 *      This tone demo example calls the main tuning repeatedly
 *      after a user specified time. In this example 1 minute
 *      is used, but by changing a define the user can change
 *      the interval between the main tuning events vFCast_Tune().
 *      Note that output is turned off during the main tuning.
 *
 *      In between main tuning the device tracks frequency with temperature
 *      by fine tuning process by repeatedly calling the vStl_SingleTxLoop()
 *      function.
 *
 *------------------------------------------------------------------------------
 *
 *    INCLUDES:
 */
#include <stdlib.h>
#include "si4010.h"
#include "si4010_api_rom.h"

#include "tone_demo.h"


/*------------------------------------------------------------------------------
 *
 *    FUNCTION DESCRIPTION:
 *      Main function for the example.
 *
 *------------------------------------------------------------------------------
 */

void    main(void)
{

//Set DMD interrupt to high priority,
// any other interrupts have to stay low priority
  PDMD=1;
// Disable the Matrix and Roff modes 
  PORT_CTRL &= ~(M_PORT_MATRIX | M_PORT_ROFF | M_PORT_STROBE);
  PORT_CTRL |=  M_PORT_STROBE;
  PORT_CTRL &= ~M_PORT_STROBE;

// LED control bit off 
  GPIO_LED = 0;

// Call the system setup. This just for initialization.
// Argument of 1 just configures the SYS module such that the
// bandgap can be turned off if needed.  
  vSys_Setup( 1 );

// Setup the bandgap for working with the temperature sensor here.
// bSys_FirstBatteryInsertWaitTime set to non zero value. 
  vSys_BandGapLdo( 1 );

// PA setup.
// fAlpha and fBeta has to be set based on antenna configuration.
// Chose a PA level and nominal cap. Both values come from
// the calculation spreadsheet. 
  rPaSetup.fAlpha      = 0.0;
  rPaSetup.fBeta       = 0.0;
  rPaSetup.bLevel      = 60;
  rPaSetup.wNominalCap = 256;
  rPaSetup.bMaxDrv     = 0;

// PA setup 
  vPa_Setup( &rPaSetup );

// ODS setup .. results from calculation spreadsheet.
// Set to 1kbps OOK 
  rOdsSetup.bModulationType = 0; // 1 .. FSK, 0 .. OOK 
  rOdsSetup.bClkDiv         = 5;
  rOdsSetup.bEdgeRate       = 0;

// Set group width to 7, which means 8 bits/encoded byte to be transmitted.
// The value must match the output width of the data encoding function
// set by the vStl_EncodeSetup() below. 
  rOdsSetup.bGroupWidth     = 7;
  rOdsSetup.wBitRate        = 4000;  // 1kbps 

// Configure the warm up intervals LC: 8, DIV: 5, PA: 4 
  rOdsSetup.bLcWarmInt  = 8;
  rOdsSetup.bDivWarmInt = 5;
  rOdsSetup.bPaWarmInt  = 4;

// ODS setup 
  vOds_Setup( &rOdsSetup );

// Set STL encoding as none. No encode function, therefore use NULL pointer 
  vStl_EncodeSetup( bEnc_NoneNrz_c, NULL );

// Generate the frame to use. Example array is fixed in CODE
// as constant for this example. Assign the beginning of the array
// to the pbFrameHead pointer. 
  pbFrameHead = (BYTE xdata *) abTone_FrameArray;

// -- Setup RTC clock to tick at 5ms intervals to implement master time 

// Setup the RTC to tick every 5ms and clear it. Keep it disabled. 
  RTC_CTRL = (0x07 << B_RTC_DIV) | M_RTC_CLR;

// Enable the RTC 
  RTC_CTRL |= M_RTC_ENA;

// Enable the RTC interrupt and global interrupt enable 
  ERTC = 1;
  EA = 1;

// Setup frequency casting .. needed to be called once per boot 
  vFCast_Setup();

// -- Loop for periodic tuning based on global time 
  while ( 1 )
  {

//  Run at 433.46 MHz.  Continue to tune frequency between frames.
//  This is not absolutely necessary as the Single transmission loop
//  also adjusts for temperature drift. It calculate values used by
//  the vStl_SingleTxLoop() during transmission. Input is in [Hz]. 
    vFCast_Tune( 433460000.0 );

//  Wait until there is a valid demodulated temperature sample 
    while ( 0 == bDmdTs_GetSamplesTaken() ) {}

//  Tune the PA with the current temperature as an argument 
    vPa_Tune( iDmdTs_GetLatestTemp() );

//  Setup the Run the Single Tx Loop 
    vStl_PreLoop( );

//  Encoding is none. First Tx byte will turn the PA on.
//  Set ODS such that it transmits all 1's output if there
//  is no data .. value 3 below. Any additional ODS_CTRL settings
//  must be done after the vStl_PreLoop() call. 
    ODS_CTRL = (ODS_CTRL & (~M_ODS_SHIFT_CTRL)) | (3 << B_ODS_SHIFT_CTRL);

//  Clear master time .. disable RTC interrupt around it.
//  Since we are not interested in master time in this
//  situation, we clear it to measure Tx time in between
//  tuning events. 
    ERTC = 0;  // Must be followed by 2 or more byte instruction 
    ERTC = 0;  // Needed at least 2 byte instruction after ERTC=0 

    vSys_SetMasterTime( 0 );

    ERTC = 1;

//  -- Keep sending the all 1's data. The constant data byte pointed
//  at by the pbFrameHead must be 0xff for all ones to keep the PA
//  activated in OOK. At least 1 byte must be supplied since only
//  when there is some data to be transmitted then the fine tuning
//  is invoked.
//  Stop sending the data after predefined time to tune.
 

//  Clear the loop termination flag and start transmitting 
    bTune = 0;
    while ( 0 == bTune )
    {

//    Keep sending the all 1's data. The constant data byte pointed
//    at by the pbFrameHead must be 0xff for all ones to keep the PA
//    activated in OOK. At least 1 byte must be supplied since only
//    when there is some data to be transmitted then the fine tuning
//    is invoked.
//    The vStl_SingleTxLoop() must be called repeatedly in the loop,
//    since it internally invokes fine frequency tuning.
 

      vStl_SingleTxLoop( pbFrameHead,
                         2 );  // Frame length in bytes 

//    Check the master time .. it is in 1ms units 
      ERTC = 0;  // Must be followed by 2 or more byte instruction 
      lTime = lSys_GetMasterTime();
      ERTC = 1;

//    Check the time. If bigger then specified timeout, then tune.
//    Divide master time by 256 .. making the time in units of 0.256s.
//    Making it a WORD gives us 4 hour and 40 minutes time 
      if ( wTuneInterval_c < ((WORD)(lTime / 256)) )
      {
        bTune = 1;
      }
    }

//  Post processing, transmission end .. tune next 
    vStl_PostLoop();
  }

// -- Infinite termination loop .. already done above 
  while ( 1 )
  {
    PCON |= M_IDLE;
    PCON = PCON;   // Must be used after going to IDLE 
  }

// This will never happen. 
  return;
}

//------------------------------------------------------------------------------


